﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Collections;
using System.IO;

namespace CreateBitmap {
    public class CBitmap {

        // Fields 
        private uint biColorImp = 0;
        private uint biColorUsed = 0;
        private uint biCompress = 0;
        private ushort biPlanes = 1;
        private uint biSizeIm = 0;
        private uint biXPixels_m = 0;
        private uint biYPixels_m = 0;
        private uint bytes_per_Row1 = 0;
        private uint bytes_per_Row2 = 0;
        private ushort colordepth = 0;
        public CHeader header = new CHeader();
        private uint infoBlock_size = 0;
        private uint pixelheight = 0;
        private uint pixelwidth = 0;
        private uint rest = 0;

        public CBitmap() {
            this.header.Filesize = 0xb2;
            this.header.BOffs = 0x3e;
            this.infoBlock_size = 40;
            this.pixelwidth = 2;
            this.pixelheight = 2;
            this.biPlanes = 1;
            this.colordepth = 1;
            this.biCompress = 0;
            this.biSizeIm = 0x74;
            this.biXPixels_m = 0;
            this.biYPixels_m = 0;
            this.biColorUsed = 0;
            this.biColorImp = 0;
            this.quadliste = new List<CRGB_Quad>();
            this.byteliste = new List<byte>();
        }

        public List<byte> byteliste;
        public List<CRGB_Quad> quadliste;


        private void calcData()     {
            int num = this.colordepth - 1;
            uint num2 = ((uint)2) << num;
            this.header.BOffs = (0x35 + (num2 << 2)) + 1;
            this.bytes_per_Row1 = (this.pixelwidth * this.colordepth) >> 3;
            if ((this.bytes_per_Row1 % 8) > 0) {
                this.bytes_per_Row1++;
            }
            this.rest = this.bytes_per_Row1 % 4;
            if (this.rest > 0) {
                this.rest = 4 - this.rest;
                this.bytes_per_Row2 = this.bytes_per_Row1 + this.rest;
            }
            else {
                this.bytes_per_Row2 = this.bytes_per_Row1;
            }
            this.biSizeIm = this.bytes_per_Row2 * this.Pixelheight;
            uint num3 = (uint)(this.quadliste.Count << 2);
            this.header.Filesize = this.header.BOffs + this.biSizeIm;
        }



        // Properties 
        public uint BiColorImp    {
            get { return this.biColorImp; }
            set { this.biColorImp = value; }
        }

        public uint BiColorUsed {
            get { return this.biColorUsed; }
            set { this.biColorUsed = value; }
        }

        public uint BiCompress {
            get { return this.biCompress; }
            set { this.biCompress = value; }
        }

        public ushort BiPlanes {
            get { return this.biPlanes; }
            set { this.biPlanes = value; }
        }

        public uint BiSizeIm {
            get { return this.biSizeIm; }
            set { this.biSizeIm = value; }
        }

        public uint BiXPixels_m {    
            get { return this.biXPixels_m; }
            set { this.biXPixels_m = value;  }
        }

        public uint BiYPixels_m {
            get { return this.biYPixels_m; }
            set { this.biYPixels_m = value; }
        }

        public ushort Colordepth {
            get { return this.colordepth; }
            set {
                switch (colordepth) {  // in Bits
                    case 1:
                        this.colordepth = value;
                        header.BOffs =14 + 40 + 2 * 4;
                        setFilesize();
                        break;
                    case 4:
                        this.colordepth = value;
                        header.BOffs =14 + 40 + 16 * 4;
                        setFilesize();
                        break;
                    case 8:
                        this.colordepth = value;
                        header.BOffs =14 + 40 + 256 * 4;
                        setFilesize();
                        break;
                    default:
                        Basis.ErrorMsg("Fehlerhafte Case-Anweisung in Methode setColordepth" + colordepth, "setColordepth");
                        break;
                }
            }
        }

        public uint InfoBlock_size {
            get { return this.infoBlock_size; }
            set { this.infoBlock_size = value; }
        }

        public uint Pixelheight {
            get { return this.pixelheight; }
            set {
                this.pixelheight = value;
                setFilesize();
            }
        }

        public uint Pixelwidth {
            get { return this.pixelwidth; }
            set {
                this.pixelwidth = value;
                setFilesize();
            }
        }

        private void setFilesize()   {
            // Filesize = Header:14    BIH:40   Farbtiefe 1:8    4:64     8:1024  + Pixeldaten
            uint summe = 0;
            summe += 14 + 40;
            switch (colordepth) {  // in Bits
                case 1:
                    summe += 8;
                    break;
                case 4:
                    summe += 64;
                    break;
                case 8:
                    summe += 1024;
                    break;
                default:
                    Basis.ErrorMsg("Fehlerhafte Case-Anweisung in Methode setFilesize" + colordepth,"setFilesize");
                    break;
            }
            uint numberofpixel = pixelwidth * colordepth;
            uint anzahlBytesProZeile = (numberofpixel) >> 3;   // 5 Bytes
            if (numberofpixel % 8 > 0) {
                anzahlBytesProZeile++;  // Rundung, der letzte Rest
            }
            // check ob Vielfaches
            uint restBytes = 0;
            uint rest = anzahlBytesProZeile % 4;
            if (rest > 0) {
                restBytes = 4 - rest;
            }

            uint gesamtanzahlBytes = pixelheight * (anzahlBytesProZeile + restBytes);
            summe += gesamtanzahlBytes;
            header.Filesize =summe;
        }

        public void save2File(string sFilename) {
            calcData();
            FileStream output = new FileStream(sFilename, FileMode.Create);
            BinaryWriter oStream = new BinaryWriter(output);
            this.header.save2File(oStream);
            this.save2FileInfoBlock(oStream);
            this.saveQuadListe(oStream);
            this.save2FileBits(oStream);
            oStream.Close();
            output.Close();
        }

        public void save2FileBits(BinaryWriter oStream) {
            foreach (byte num4 in this.byteliste) {
                oStream.Write(num4);
            }
        }

        private void saveQuadListe(BinaryWriter oStream)  {
            foreach (CRGB_Quad quad in this.quadliste) {
                quad.save2File(oStream);
            }

        }

        private void save2FileInfoBlock(BinaryWriter oStream) {
            oStream.Write(this.infoBlock_size);
            oStream.Write(this.pixelwidth);
            oStream.Write(this.pixelheight);
            oStream.Write(this.biPlanes);
            oStream.Write(this.colordepth);
            oStream.Write(this.biCompress);
            oStream.Write(this.biSizeIm);
            oStream.Write(this.biXPixels_m);
            oStream.Write(this.biYPixels_m);
            oStream.Write(this.biColorUsed);
            oStream.Write(this.biColorImp);
        }


    }
}


